<?php
/* --------------------------------------------------------------
 LanguageSpecificConfiguration.php 2020-03-09
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 --------------------------------------------------------------
 */

declare(strict_types=1);

namespace Gambio\Core\Configuration\Models\Write;

/**
 * Class LanguageSpecificConfiguration
 * @package Gambio\Core\Configuration\Models\Write
 */
class LanguageSpecificConfiguration implements Interfaces\LanguageSpecificConfiguration
{
    /**
     * @var string
     */
    private $key;
    
    /**
     * @var string
     */
    private $value;
    
    /**
     * @var string
     */
    private $languageCode;
    
    
    /**
     * Configuration constructor.
     *
     * @param string $key
     * @param string $value
     * @param string $languageCode
     */
    private function __construct(string $key, string $value, string $languageCode)
    {
        $this->key          = $key;
        $this->value        = $value;
        $this->languageCode = $languageCode;
    }
    
    
    /**
     * Factory method to create Configuration.
     *
     * @param string $key
     * @param string $value
     * @param string $languageCode
     *
     * @return static
     */
    public static function create(string $key, string $value, string $languageCode): self
    {
        return new static($key, $value, $languageCode);
    }
    
    
    /**
     * @inheritDoc
     */
    public function key(): string
    {
        return $this->key;
    }
    
    
    /**
     * @inheritDoc
     */
    public function value(): string
    {
        return $this->value;
    }
    
    
    /**
     * @inheritDoc
     */
    public function languageCode(): string
    {
        return $this->languageCode;
    }
}