<?php
/*------------------------------------------------------------------------------
 OnGetSellingUnitAvailableQuantityListener.php 2020-10-27
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 -----------------------------------------------------------------------------*/

declare(strict_types=1);

namespace Gambio\Shop\Properties\Database\Listeners;

use Gambio\Shop\Properties\Database\Services\Interfaces\PropertyQuantityReadServiceInterface;
use Gambio\Shop\SellingUnit\Database\Unit\Events\Interfaces\OnGetSellingUnitAvailableQuantityEventInterface;

/**
 * Class OnGetSellingUnitAvailableQuantityListener
 * @package Gambio\Shop\Properties\Database\Listeners
 */
class OnGetSellingUnitAvailableQuantityListener
{
    /**
     * @var PropertyQuantityReadServiceInterface
     */
    private $service;
    
    
    /**
     * OnGetSellingUnitQuantityListener constructor.
     *
     * @param PropertyQuantityReadServiceInterface $service
     */
    public function __construct(
        PropertyQuantityReadServiceInterface $service
    ) {
        $this->service = $service;
    }
    
    
    /**
     * @param OnGetSellingUnitAvailableQuantityEventInterface $event
     *
     * @return OnGetSellingUnitAvailableQuantityEventInterface
     */
    public function __invoke(OnGetSellingUnitAvailableQuantityEventInterface $event
    ): OnGetSellingUnitAvailableQuantityEventInterface {
        
        $quantity = $this->service->getAvailableQuantityBy($event->id(), $event->product(), $event->requested());
        if ($quantity) {
            $event->addQuantity($quantity);
            $event->stopPropagation();
        }
        
        return $event;
    }
    
}
