<?php
/*------------------------------------------------------------------------------
 CachedPropertyReader.php 2020-10-26
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 -----------------------------------------------------------------------------*/


namespace Gambio\Shop\Properties\Database\Readers;


use Gambio\Shop\Product\ValueObjects\ProductId;
use Gambio\Shop\Properties\Properties\Collections\CombinationCollectionInterface;
use Gambio\Shop\SellingUnit\Unit\ValueObjects\SellingUnitId;

class CachedPropertyReader extends PropertyReader
{
    protected $hasSurcharge              = [];
    protected $getCombinationsFor        = [];
    protected $hasNonLinearSurcharge     = [];
    protected $hasProperties             = [];
    protected $getCombinationModifierIds = [];
    protected   $getNumberOfProperties = [];
    
    
    /**
     * @inheritDoc
     */
    protected function hasSurcharge(int $productId): bool
    {
        if (!isset($this->hasSurcharge[$productId])) {
            $this->hasSurcharge[$productId] = parent::hasSurcharge($productId);
        }
        return $this->hasSurcharge[$productId];

    }

    /**
     * @inheritDoc
     */
    public function hasProperties(ProductId $productId): bool
    {
        if (!isset($this->hasProperties[$productId->value()])) {
            $this->hasProperties[$productId->value()] = parent::hasProperties($productId);
        }
        return $this->hasProperties[$productId->value()];

    }

    /**
     * @inheritDoc
     */
    public function getCombinationModifierIds(int $combinationId): iterable
    {
        if (!isset($this->getCombinationModifierIds[$combinationId])) {
            $this->getCombinationModifierIds[$combinationId] = parent::getCombinationModifierIds($combinationId);
        }
        return $this->getCombinationModifierIds[$combinationId];

    }

    /**
     * @inheritDoc
     */
    public function getCombinationsFor(SellingUnitId $id): CombinationCollectionInterface
    {
        if (!isset($this->getCombinationsFor[$id->hash()])) {
            $this->getCombinationsFor[$id->hash()] = parent::getCombinationsFor($id);
        }
        return $this->getCombinationsFor[$id->hash()];
    }

    /**
     * @inheritDoc
     */
    protected function hasNonLinearSurcharge(int $productId): bool
    {
        $key = $productId;
        if (!isset($this->hasNonLinearSurcharge[$key])) {
            $this->hasNonLinearSurcharge[$key] = parent::hasNonLinearSurcharge($productId);
        }
        return $this->hasNonLinearSurcharge[$key];

    }
    
    protected function getNumberOfProperties(ProductId $productId): int {
        $key = $productId->value();
        if (!isset($this->getNumberOfProperties[$key])) {
            $this->getNumberOfProperties[$key] = parent::getNumberOfProperties($productId);
        }
        return $this->getNumberOfProperties[$key];
    }

}