<?php
/*------------------------------------------------------------------------------
 ServiceProvider.php 2020-10-29
 Gambio GmbH
 http://www.gambio.de
 Copyright (c) 2020 Gambio GmbH
 Released under the GNU General Public License (Version 2)
 [http://www.gnu.org/licenses/gpl-2.0.html]
 -----------------------------------------------------------------------------*/

namespace Gambio\Shop\Properties\Database;

use Doctrine\DBAL\Connection;
use Gambio\Core\Event\EventListenerProvider;
use Gambio\Shop\Properties\Database\Criterias\CheckStockBeforeShoppingCartCriteria;
use Gambio\Shop\Properties\Database\Readers\CachedPropertyReader;
use Gambio\Shop\Properties\Database\Readers\Interfaces\PropertyReaderInterface;
use Gambio\Shop\Properties\Database\Repositories\Interfaces\PropertyReadRepositoryInterface;
use Gambio\Shop\Properties\Database\Repositories\PropertyReadRepository;
use Gambio\Shop\Properties\Database\Services\Interfaces\PropertiesReaderServiceInterface;
use Gambio\Shop\Properties\Database\Services\Interfaces\PropertyQuantityReadServiceInterface;
use Gambio\Shop\Properties\Database\Services\PropertiesReaderService;
use Gambio\Shop\Properties\Database\Services\PropertyQuantityReadService;
use Gambio\Shop\Properties\Database\Listeners\OnGetSellingUnitAvailableQuantityListener;
use Gambio\Shop\Properties\Properties\Builders\CombinationBuilder;
use Gambio\Shop\Properties\Properties\Builders\CombinationBuilderInterface;
use Gambio\Shop\SellingUnit\Database\Unit\Events\OnGetSellingUnitAvailableQuantityEvent;
use League\Container\ServiceProvider\AbstractServiceProvider;
use League\Container\ServiceProvider\BootableServiceProviderInterface;

class ServiceProvider extends AbstractServiceProvider  implements BootableServiceProviderInterface
{
    /**
     * @var array
     */
    protected $provides = [
        PropertiesReaderServiceInterface::class,
        CheckStockBeforeShoppingCartCriteria::class,
        OnGetSellingUnitAvailableQuantityListener::class
    ];
    
    /**
     * @inheritDoc
     */
    public function register()
    {
        $this->container->share(CheckStockBeforeShoppingCartCriteria::class)
            ->addArgument(defined('STOCK_ALLOW_CHECKOUT') ? STOCK_ALLOW_CHECKOUT === 'true' : false)
            ->addArgument(defined('STOCK_CHECK') ? STOCK_CHECK === 'true' : false)
            ->addArgument(defined('CHECK_STOCK_BEFORE_SHOPPING_CART') ? CHECK_STOCK_BEFORE_SHOPPING_CART === 'true' : false)
            ->addArgument(defined('ATTRIBUTE_STOCK_CHECK') ? ATTRIBUTE_STOCK_CHECK === 'true' : false);
        
        
        $this->container->share(PropertyQuantityReadServiceInterface::class, PropertyQuantityReadService::class)
            ->addArgument(PropertiesReaderServiceInterface::class)
            ->addArgument(CheckStockBeforeShoppingCartCriteria::class)
            ->addArgument(defined('STOCK_CHECK') ? STOCK_CHECK === 'true' : false)
            ->addArgument(defined('ATTRIBUTE_STOCK_CHECK') ? ATTRIBUTE_STOCK_CHECK === 'true' : false);
    
        $this->container->share(OnGetSellingUnitAvailableQuantityListener::class)
            ->addArgument(PropertyQuantityReadServiceInterface::class, PropertyQuantityReadService::class);

        $this->container->share(CombinationBuilderInterface::class, CombinationBuilder::class);

        $this->container->share(PropertiesReaderServiceInterface::class, PropertiesReaderService::class)
                        ->addArgument(PropertyReadRepositoryInterface::class);
        
        $this->container->share(PropertyReadRepositoryInterface::class, PropertyReadRepository::class)
                        ->addArgument(PropertyReaderInterface::class);
        
        $this->container->share(PropertyReaderInterface::class, CachedPropertyReader::class)
            ->addArgument(Connection::class)
            ->addArgument(CombinationBuilderInterface::class)
            ->addArgument(CheckStockBeforeShoppingCartCriteria::class);
        
        $this->container->share(PropertiesReaderServiceInterface::class, PropertiesReaderService::class);
        $this->container->share(PropertiesReaderService::class);
    }
    
    public function boot()
    {
        /** @var EventListenerProvider $listenerProvider */
        $listenerProvider = $this->container->get(EventListenerProvider::class);
        $listenerProvider->attachListener(OnGetSellingUnitAvailableQuantityEvent::class, OnGetSellingUnitAvailableQuantityListener::class);
    }
}