<?php
/* --------------------------------------------------------------
   dependent.inc.php 2020-09-10
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2020 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/** @var DatabaseModel $this */

use Doctrine\DBAL\Connection;
use Gambio\Core\Configuration\Migration\ConfigurationMigrationAssistant;
use Gambio\Core\Configuration\Migration\ConfigurationMigrations;
use League\Container\Container;

require_once __DIR__ . '/../../classes/Migration/Configuration/autoload.php';

/*
 * UPDATE WITHDRAWALS DATABASE TABLE AND MIGRATE GENDER
 */
if($this->columnExists('withdrawals', 'order_id')){
    $query = 'ALTER TABLE `withdrawals` MODIFY `order_id` int(11) NULL';
    $t_success &= $this->query($query, true);
}
if($this->columnExists('withdrawals','customer_id')){
    $query = 'ALTER TABLE `withdrawals` MODIFY `customer_id` int(11) NULL';
    $t_success &= $this->query($query, true);
}
if($this->columnExists('withdrawals','customer_gender')){
    $query = "UPDATE `withdrawals` SET `customer_gender` = 'm' WHERE `customer_gender` = 'Herr';";
    $t_success &= $this->query($query, true);
    $query = "UPDATE `withdrawals` SET `customer_gender` = 'm' WHERE `customer_gender` = 'Mr';";
    $t_success &= $this->query($query, true);
    $query = "UPDATE `withdrawals` SET `customer_gender` = 'f' WHERE `customer_gender` = 'Frau';";
    $t_success &= $this->query($query, true);
    $query = "UPDATE `withdrawals` SET `customer_gender` = 'f' WHERE `customer_gender` = 'Miss/Ms/Mrs';";
    $t_success &= $this->query($query, true);
    $query = "UPDATE `withdrawals` SET `customer_gender` = 'd' WHERE `customer_gender` != 'm' AND `customer_gender` != 'f';";
    $t_success &= $this->query($query, true);
    
    $query = "ALTER TABLE `withdrawals` MODIFY `customer_gender` varchar(1) NOT NULL DEFAULT 'd'";
    $t_success &= $this->query($query, true);
}
if($this->columnExists('withdrawals','order_date')){
    $query = 'ALTER TABLE `withdrawals` MODIFY `order_date` datetime NULL';
    $t_success &= $this->query($query, true);
}
if($this->columnExists('withdrawals','delivery_date')){
    $query = 'ALTER TABLE `withdrawals` MODIFY `delivery_date` datetime NULL';
    $t_success &= $this->query($query, true);
}

/*
 * ADD NEW ENUM VALUE FOR ADMIN ACCESS GROUP ITEM TYPE
 */
$query = 'ALTER TABLE `admin_access_group_items` MODIFY `type` ENUM(\'CONTROLLER\', \'PAGE\', \'AJAX_HANDLER\', \'ROUTE\') DEFAULT \'CONTROLLER\' NOT NULL;';
$t_success &= $this->query($query, true);

/*
 * REMOVE ALL ADMIN ACCESS GROUP ITEMS WITH TYPE "ROUTE"
 */
$query = 'DELETE FROM `admin_access_group_items` WHERE `type` = "ROUTE";';
$t_success &= $this->query($query, true);

/*
 * ADD NEW WITHDRAWALS CONTROLLER AND ROUTES TO ADMIN ACCESS SERVICE
 */
$controllerGroupId = $this->getAdminAccessGroupIdByIdentifier('CONTROLLER', 'Withdrawals');
if($controllerGroupId === false){
    $pageGroupId = $this->getAdminAccessGroupIdByIdentifier('PAGE', 'withdrawals.php');
    $this->addAdminAccessGroupItem($pageGroupId,'CONTROLLER', 'Withdrawals');
}
$controllerGroupId = $this->getAdminAccessGroupIdByIdentifier('ROUTE', '/api.php/v3/withdrawals');
if($controllerGroupId === false){
    $pageGroupId = $this->getAdminAccessGroupIdByIdentifier('PAGE', 'withdrawals.php');
    $this->addAdminAccessGroupItem($pageGroupId,'ROUTE', '/api.php/v3/withdrawals');
}

/*
 * ADD NEW PARCEL SERVICE ROUTES TO ADMIN ACCESS SERVICE
 */
$routesToAdd = [
    '/api.php/v3/parcel-services',
    '/admin/parcel-services',
    '/admin/api/parcel-services',
    '/admin/api/parcel-services/{id:[0-9]+}',
];
foreach($routesToAdd as $route){
    $controllerGroupId = $this->getAdminAccessGroupIdByIdentifier('ROUTE', $route);
    if($controllerGroupId === false){
        $pageGroupId = $this->getAdminAccessGroupIdByIdentifier('PAGE', 'parcel_services.php');
        $this->addAdminAccessGroupItem($pageGroupId,'ROUTE', $route);
    }
}

/*
 * ADD NEW CONFIGURATION ROUTES TO ADMIN ACCESS SERVICE
 */
$controllerGroupId = $this->getAdminAccessGroupIdByIdentifier('ROUTE', '/admin/configuration/save');
if($controllerGroupId === false){
    $pageGroupId = $this->getAdminAccessGroupIdByIdentifier('PAGE', 'configuration.php');
    $this->addAdminAccessGroupItem($pageGroupId,'ROUTE', '/admin/configuration/save');
}

$query  = 'SHOW COLUMNS FROM `products_options` LIKE \'products_option_display_type\';';
$result = $this->query($query, true);

if ($result->num_rows < 1) {
    
    $query = 'ALTER TABLE `products_options` ADD `products_option_display_type` VARCHAR(64) NOT NULL DEFAULT "Dropdown" AFTER `products_options_name`;';
    $t_success &= $this->query($query, true);
}

$query  = 'SHOW COLUMNS FROM `properties` LIKE \'display_type\';';
$result = $this->query($query, true);

if ($result->num_rows < 1) {
    
    $query = 'ALTER TABLE `properties` ADD `display_type` VARCHAR(64) NOT NULL DEFAULT \'Dropdown\' AFTER `sort_order`;';
    $t_success &= $this->query($query, true);
}
/**
 * display_image
 */
$query  = 'SHOW COLUMNS FROM `properties_values` LIKE \'display_image\';';
$result = $this->query($query, true);

if ($result->num_rows < 1) {

    $query = 'ALTER TABLE `properties_values` ADD `display_image` VARCHAR(256) NULL AFTER `value_price`;';
    $t_success &= $this->query($query, true);
}

$productImageListTableExists = 'SHOW TABLES LIKE \'product_image_list\';';
$row = $this->query($productImageListTableExists, true);

if ($row->num_rows < 1) {
    
    $query = 'CREATE TABLE `product_image_list` (
                `product_image_list_id` int(11) NOT NULL AUTO_INCREMENT,
                `product_image_list_name` varchar(255) NOT NULL,
                PRIMARY KEY (`product_image_list_id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8;';

    $t_success &= $this->query($query, true);
}

$productImageListImageTableExists = 'SHOW TABLES LIKE \'product_image_list_image\';';
$row = $this->query($productImageListImageTableExists, true);

if ($row->num_rows < 1) {
    
    $query = 'CREATE TABLE `product_image_list_image` (
                `product_image_list_image_id` int(11) NOT NULL AUTO_INCREMENT,
                `product_image_list_id` int(11) NOT NULL,
                `product_image_list_image_local_path` varchar(255) NOT NULL,
                `product_image_list_image_sort_order` int(8) NOT NULL DEFAULT \'0\',
                PRIMARY KEY (`product_image_list_image_id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8;';

    $t_success &= $this->query($query, true);
}

$productImageListImageTextTableExists = 'SHOW TABLES LIKE \'product_image_list_image_text\';';
$row = $this->query($productImageListImageTextTableExists, true);

if ($row->num_rows < 1) {
    
    $query = 'CREATE TABLE `product_image_list_image_text` (
                `product_image_list_image_id` int(11) NOT NULL,
                `product_image_list_image_text_type` varchar(16) NOT NULL,
                `product_image_list_image_text_value` varchar(255) NOT NULL,
                `language_id` int(11) NOT NULL DEFAULT \'0\',
                PRIMARY KEY (`product_image_list_image_id`, `language_id`, `product_image_list_image_text_type`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8;';

    $t_success &= $this->query($query, true);
}

$productImageListCombiTableExists = 'SHOW TABLES LIKE \'product_image_list_combi\';';
$row = $this->query($productImageListCombiTableExists, true);

if ($row->num_rows < 1) {
    
    $query = 'CREATE TABLE `product_image_list_combi` (
                `products_properties_combis_id` int(11) NOT NULL,
                `product_image_list_id` int(11) NOT NULL,
                PRIMARY KEY (`products_properties_combis_id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8;';

    $t_success &= $this->query($query, true);
}

$productImageListAttributeTableExists = 'SHOW TABLES LIKE \'product_image_list_attribute\';';
$row = $this->query($productImageListAttributeTableExists, true);

if ($row->num_rows < 1) {
    
    $query = 'CREATE TABLE `product_image_list_attribute` (
                `products_attributes_id` int(11) NOT NULL,
                `product_image_list_id` int(11) NOT NULL,
                PRIMARY KEY (`products_attributes_id`)
            ) ENGINE=InnoDB DEFAULT CHARSET=utf8;';

    $t_success &= $this->query($query, true);
}

/**
 * Combi_image migration
 */
$query  = 'SHOW COLUMNS FROM `products_properties_combis` LIKE \'combi_image\';';
$result = $this->query($query, true);

if ($result->num_rows === 1) {
    
    $relativePathToCombiImages  = 'images/product_images/properties_combis_images/';
    
    $query = 'SELECT `products_properties_combis_id`, `combi_image` FROM `products_properties_combis` WHERE `combi_image` != ""';
    $result = $this->query($query, true);
    
    
    if ($result->num_rows > 0) {
        
        $imageLists    = [];
        $listIdCounter = 0;
        
        /*
         * Collecting combi image data
         */
        while($row = $result->fetch_assoc()) {
            [
                'combi_image'                   => $combiImage,
                'products_properties_combis_id' => $combiId
            ] = $row;
            
            
            if (!isset($imageLists[$combiImage])) {
                
                $imageLists[$combiImage] = [
                    'listName' => preg_replace('#(.*)\.\w+$#', '$1', $combiImage),
                    'listId'   => ++$listIdCounter,
                    'image'    => $combiImage,
                    'combiIds' => [(int)$combiId]
                ];
                
            } else {
                
                $imageLists[$combiImage]['combiIds'][] = (int)$combiId;
            }
        }
        
        /**
         * collecting available admin languages
         */
        $languageIds    = [];
        $languagesQuery = 'SELECT `languages_id` FROM `languages` WHERE `status_admin` = 1';
        $result         = $this->query($languagesQuery, true);
        
        while($row = $result->fetch_assoc()) {
            
            $languageIds[] = (int)$row['languages_id'];
        }
        
        /**
         * Inserting new product image lists
         */
        $insertImageListsQuery = 'INSERT INTO `product_image_list`(`product_image_list_id`, `product_image_list_name`) VALUES ';
        
        foreach ($imageLists as $imageList) {
            
            $insertImageListsQuery .= '(' . $imageList['listId'] . ', "' . $imageList['listName'] . '"), ';
        }
        
        $insertImageListsQuery = substr($insertImageListsQuery, 0, -2) . ';';
        $t_success &= (bool)$this->query($insertImageListsQuery);
        
        /**
         * Inserting new product image list images
         */
        $insertImageListImagesQuery = 'INSERT INTO `product_image_list_image`(`product_image_list_image_id`, `product_image_list_id`, `product_image_list_image_local_path`, `product_image_list_image_sort_order`) VALUES ';
        
        foreach ($imageLists as $imageList) {
            
            $insertImageListImagesQuery .= '(' . $imageList['listId'] . ', ' . $imageList['listId'] . ', "' . $relativePathToCombiImages . $imageList['image'] . '", 0), ';
        }
        
        $insertImageListImagesQuery = substr($insertImageListImagesQuery, 0, -2) . ';';
        $t_success &= (bool)$this->query($insertImageListImagesQuery);
        
        /**
         * Inserting new product image list image texts
         */
        $insertImageListImageTextsQuery = 'INSERT INTO `product_image_list_image_text`(`product_image_list_image_id`, `product_image_list_image_text_type`, `product_image_list_image_text_value`, `language_id`) VALUES ';
        
        foreach ($imageLists as $imageList) {
            
            foreach ($languageIds as $languageId) {
                
                $insertImageListImageTextsQuery .= '(' . $imageList['listId'] . ', "alt_title", "", ' . $languageId . '), ';
                $insertImageListImageTextsQuery .= '(' . $imageList['listId'] . ', "title", "", ' . $languageId . '), ';
            }
        }
        
        $insertImageListImageTextsQuery = substr($insertImageListImageTextsQuery, 0, -2) . ';';
        $t_success &= $this->query($insertImageListImageTextsQuery, true);
        
        /**
         * Assigning new list to existing combinations
         */
        $insertImageListCombiQuery = 'INSERT INTO `product_image_list_combi`(`products_properties_combis_id`, `product_image_list_id`) VALUES ';
        
        foreach ($imageLists as $imageList) {
            
            foreach ($imageList['combiIds'] as $combiId) {
                
                $insertImageListCombiQuery .= '(' . $combiId . ', ' . $imageList['listId'] . '), ';
            }
        }
        
        $insertImageListCombiQuery = substr($insertImageListCombiQuery, 0, -2) . ';';
        $t_success &= $this->query($insertImageListCombiQuery, true);
    }
    
    /**
     * Dropping 'combi_image' column from `products_properties_combis`
     */
    $dropCombiImageColumnQuery = 'ALTER TABLE `products_properties_combis` DROP COLUMN `combi_image`;';
    $t_success &= $this->query($dropCombiImageColumnQuery, true);
}


$exportSchemeFieldPropertyImageListFieldExistsQuery = 'SELECT `field_id` FROM `export_scheme_fields` WHERE `field_name` = "p_property_image_list_id" AND `created_by` = "gambio" AND `scheme_id` = 5;';
$propertyImageListFieldExistsResult                 = $this->query($exportSchemeFieldPropertyImageListFieldExistsQuery, true);
if ($propertyImageListFieldExistsResult->num_rows < 1) {
    
    $fieldSortOrderQuery       = 'SELECT MAX(`sort_order`) as "sortOrder" FROM `export_scheme_fields` WHERE `scheme_id` = 5;';
    $fieldSortOrderResult      = $this->query($fieldSortOrderQuery, true);
    $fieldSortOrder            = (int)current($fieldSortOrderResult->fetch_assoc()) + 1;
    $imageListFieldInsertQuery = '
        INSERT INTO `export_scheme_fields`
            (`field_id`, `scheme_id`, `field_name`, `field_content`, `field_content_default`, `created_by`, `sort_order`, `status`)
        VALUES
           (NULL, 5, "p_property_image_list_id", "{p_property_image_list_id}", "", "gambio", ' . $fieldSortOrder . ', 1);';

    $t_success &= (bool)$this->query($imageListFieldInsertQuery);
}

/**
 * Removing the `properties_column` combi_image from the `export_scheme_properties`
 */
$combiImageEntryInSchemePropertiesExistsQuery = 'SELECT `sort_order` FROM `export_scheme_properties` WHERE `properties_column` = "combi_image" AND `scheme_id` = 5';
$combiImageEntryInSchemeExistsResult          = $this->query($combiImageEntryInSchemePropertiesExistsQuery, true);

if ($combiImageEntryInSchemeExistsResult->num_rows !== 0) {
    
    $sortOrder = (int)current($combiImageEntryInSchemeExistsResult->fetch_assoc());
    
    $deleteCombiImageRowQuery = 'DELETE FROM `export_scheme_properties` WHERE `properties_column` = "combi_image" AND `scheme_id` = 5';
    $t_success &= $this->query($deleteCombiImageRowQuery, true);
    
    $updateSchemePropertiesSortOrderQuery = '
        UPDATE `export_scheme_properties`
        SET `sort_order` = `sort_order` - 1
        WHERE `scheme_id` = 5
        AND `sort_order` > ' . $sortOrder;
    $t_success &= is_numeric($this->query($updateSchemePropertiesSortOrderQuery));
}

if (!$this->table_exists('gx_configurations')) {
    $query = "CREATE TABLE IF NOT EXISTS `gx_configurations` (
	`id`              int(11)      NOT NULL AUTO_INCREMENT,
	`key`             varchar(255) NOT NULL,
	`language_id`     int(11)           DEFAULT NULL,
	`value`           mediumtext,
	`default`         mediumtext,
	`type`            varchar(255)      DEFAULT NULL,
	`sort_order`      int(5)            DEFAULT NULL,
	`legacy_group_id` int(11)           DEFAULT NULL,
	`last_modified`   timestamp    NULL DEFAULT CURRENT_TIMESTAMP,
	PRIMARY KEY (`id`),
	UNIQUE KEY `uc_cfg_key`(`key`, `language_id`),
	KEY `key`(`key`)
)
	ENGINE = InnoDB
	DEFAULT CHARSET = `utf8`";

    $t_success &= $this->query($query, true);
}


$container = new Container();
$container->addServiceProvider(DoctrineConnectionServiceProvider::class);
$connection = $container->get(Connection::class);

$configurationMigration = new ConfigurationMigrations($connection, new ConfigurationMigrationAssistant($connection));
$configurationMigration->migrate();


$t_success &= is_numeric($this->query("UPDATE `gx_configurations` SET `language_id` = NULL WHERE `key` IN ('configuration/MODULE_ORDER_TOTAL_COD_FEE_TRANSFER_CHARGE', 'configuration/MODULE_ORDER_TOTAL_COD_FEE_RULES')"));


// deleting duplicates in gx_configuration table
$query = 'SELECT 
    `id`, 
    `key`
FROM
    `gx_configurations`
GROUP BY
    `key`, `language_id`
HAVING 
    COUNT(*) > 1';

$duplicates = $this->query($query);
foreach ($duplicates as $row) {
    $query     = 'DELETE FROM `gx_configurations` 
                    WHERE 
                        `key` = "' . $this->real_escape_string($row['key']) . '" AND 
                        `id` != ' . $row['id'];
    $t_success &= $this->query($query, true);
}

function insertGxConfiguration($key, $value, $dbModel, &$t_success)
{
    $check = $dbModel->query('SELECT * FROM `gx_configurations` WHERE `key` = "' . $dbModel->real_escape_string($key)
                             . '"',
                             true);
    if ($check->num_rows < 1) {
        $t_success &= (bool)$dbModel->query('INSERT INTO `gx_configurations` 
                                                SET 
                                                    `key` = "' . $dbModel->real_escape_string($key) . '", 
                                                    `value` = "' . $dbModel->real_escape_string($value) . '"');
    }
}


if (!$this->columnExists('coupon_gv_queue', 'orders_products_id')) {
    $t_success &= $this->query('ALTER TABLE `coupon_gv_queue` ADD `orders_products_id` INT NULL DEFAULT NULL AFTER `order_id`',
                               true);
}

if (!$this->table_exists('orders_products_to_coupons')) {
    $t_success &= $this->query('CREATE TABLE `orders_products_to_coupons` (
                                          `orders_products_id` int(11) NOT NULL,
                                          `coupon_id` int(11) NOT NULL
                                        ) ENGINE=InnoDB DEFAULT CHARSET=`utf8`;',
                               true);
    $t_success &= $this->query('ALTER TABLE `orders_products_to_coupons` 
                                        ADD UNIQUE KEY `orders_products_id` (`orders_products_id`,`coupon_id`)',
                               true);
}

$gsCheckQuery        = $this->query("SELECT `value` FROM `gx_configurations` WHERE `key` = 'configuration/ACTIVATE_GIFT_SYSTEM'");
$gsCheckRow          = $gsCheckQuery[0];
$giftSystemInstalled = filter_var($gsCheckRow['value'], FILTER_VALIDATE_BOOLEAN);
insertGxConfiguration('gm_configuration/MODULE_CENTER_GIFTSYSTEM_INSTALLED', $giftSystemInstalled ? '1' : '0', $this, $t_success);
insertGxConfiguration('extensions/giftsystem/releaseOrderStatuses', '', $this, $t_success);

/** @var mysqli_result $securityCodeQuery */
$securityCodeQuery  = $this->query("SELECT `value` FROM `gx_configurations` WHERE `key` = 'configuration/SECURITY_CODE_LENGTH'");
$securityCodeLength = (int)$securityCodeQuery[0]['value'];
$securityCodeLength = min(max(5, $securityCodeLength), 16);
insertGxConfiguration('extensions/giftsystem/securityCodeLength', (string)$securityCodeLength, $this, $t_success);

/** @var mysqli_result $newSignupGiftVoucherAmountQuery */
$newSignupGiftVoucherAmountQuery = $this->query("SELECT `value` FROM `gx_configurations` WHERE `key` = 'configuration/NEW_SIGNUP_GIFT_VOUCHER_AMOUNT'");
$newSignupGiftVoucherAmount      = (float)$newSignupGiftVoucherAmountQuery[0]['value'];
insertGxConfiguration('extensions/giftsystem/newSignupGiftVoucherAmount', (string)$newSignupGiftVoucherAmount, $this, $t_success);

/** @var mysqli_result $newSignupGiftVoucherAmountQuery */
$newSignupDiscountCouponQuery = $this->query("SELECT `value` FROM `gx_configurations` WHERE `key` = 'configuration/NEW_SIGNUP_DISCOUNT_COUPON'");
$newSignupDiscountCoupon      = $newSignupDiscountCouponQuery[0]['value'];
insertGxConfiguration('extensions/giftsystem/newSignupDiscountCoupon', (string)$newSignupDiscountCoupon, $this, $t_success);

$t_success &= is_numeric($this->query("UPDATE gx_configurations SET legacy_group_id = 0 WHERE `key` IN
('configuration/ACTIVATE_GIFT_SYSTEM', 'configuration/SECURITY_CODE_LENGTH', 'configuration/NEW_SIGNUP_GIFT_VOUCHER_AMOUNT', 'configuration/NEW_SIGNUP_DISCOUNT_COUPON')"));

$askAboutProductEmailQuery  = 'SELECT `value` FROM `gx_configurations` WHERE `key` = "gm_configuration/GM_TELL_A_FRIEND_USE_CUSTOMER_EMAIL"';
$askAboutProductEmailResult = $this->query($askAboutProductEmailQuery, true);

if ($askAboutProductEmailResult->num_rows < 1) {
    
    $askAboutProductEmailInsertQuery = 'INSERT INTO `gx_configurations` (`key`, `language_id`, `value`, `default`)
    VALUES ("gm_configuration/GM_TELL_A_FRIEND_USE_CUSTOMER_EMAIL", NULL, "true", "true")';

    $t_success &= (bool)$this->query($askAboutProductEmailInsertQuery);
}

$displayOfPropertyCombinationSelectionQuery  = 'SELECT `value` FROM `gx_configurations` WHERE `key` = "gm_configuration/DISPLAY_OF_PROPERTY_COMBINATION_SELECTION"';
$displayOfPropertyCombinationSelectionResult = $this->query($displayOfPropertyCombinationSelectionQuery, true);

if ($displayOfPropertyCombinationSelectionResult->num_rows < 1) {
    
    $insertQuery = 'INSERT INTO `gx_configurations` (`key`, `language_id`, `value`, `default`)
    VALUES ("gm_configuration/DISPLAY_OF_PROPERTY_COMBINATION_SELECTION", NULL, "DISPLAY_PROPERTY_PLUS_ARTICLE_IMAGES", "DISPLAY_ONLY_PROPERTY_IMAGES")';

    $t_success &= (bool)$this->query($insertQuery);
}

/**
 * Add Hermes HSI Parcel Service
 */
$t_check = $this->query('SELECT * FROM `parcel_services_description` WHERE `url` LIKE "%https://www.myhermes.de/empfangen/sendungsverfolgung/suchen/sendungsinformation%"', true);
if($t_check->num_rows == 0)
{
    $query        = 'INSERT INTO `parcel_services` (`name`, `default`) VALUES ("Hermes HSI", 0)';
    $lastInsertId = $this->query($query);
    $t_success &= (boolean)$lastInsertId;

    $query     = 'INSERT INTO `parcel_services_description` (`parcel_service_id`, `language_id`, `url`, `comment`) VALUES ';
    $languages = $this->getLanguages();

    foreach ($languages as $language) {
        if ($language['languages_id'] !== '2') {
            $query .= '(' . $lastInsertId . ', ' . $language['languages_id']
                      . ', "https://www.myhermes.de/empfangen/sendungsverfolgung/suchen/sendungsinformation#{TRACKING_NUMBER}", "You can access the shipment tracking for your order by visiting the link above."),';
        } else {
            $query .= '(' . $lastInsertId . ', ' . $language['languages_id']
                      . ', "https://www.myhermes.de/empfangen/sendungsverfolgung/suchen/sendungsinformation#{TRACKING_NUMBER}", "Die Sendungsverfolgung für Ihre Bestellung können Sie über den oben stehenden Link aufrufen."),';
        }
    }

    // remove last ,
    $query = substr($query, 0, -1);

    $t_success &= (boolean)$this->query($query, true);
}

/**
 * Repair charset of url keyword tables
 */
$tables = ['products_description', 'categories_description', 'content_manager'];

foreach ($tables as $table) {
    
    $changeCollationQuery = 'ALTER TABLE ' . $table . '  MODIFY gm_url_keywords VARCHAR(255) CHARACTER SET latin1 COLLATE latin1_general_cs NOT NULL DEFAULT "";';
    $t_success &= $this->query($changeCollationQuery, true);
}

$checkIfFreeContent1MalibuExists = $this->query('SELECT * FROM `content_manager_aliases` WHERE `content_alias`="Malibu-below-slider"', true);
if ($checkIfFreeContent1MalibuExists->num_rows < 1) {
    
    $nextContentGroupIdQuery  = 'SELECT MAX(`content_group`) + 1 as "next_cm_id" FROM `content_manager`';
    $nextContentGroupIdResult = $this->query($nextContentGroupIdQuery, true);
    $nextContentGroupId       = (int)$nextContentGroupIdResult->fetch_assoc()['next_cm_id'];
    $contentEn                = $this->query('SELECT `content_text`, `content_status`, `group_ids` FROM `content_manager` WHERE `content_group` = 4321002 AND `languages_id` = 1')[0];
    $contentDe                = $this->query('SELECT `content_text`, `content_status`, `group_ids` FROM `content_manager` WHERE `content_group` = 4321002 AND `languages_id` = 2')[0];

    $createContent = "INSERT INTO `content_manager` (`categories_id`, `parent_id`, `group_ids`, `languages_id`, `content_name`, `content_title`, `content_heading`, `content_text`, `sort_order`, `file_flag`, `content_file`, `download_file`, `content_status`, `content_group`, `content_delete`, `gm_link`, `gm_link_target`, `gm_priority`, `gm_changefreq`, `gm_sitemap_entry`, `gm_robots_entry`, `gm_url_keywords`, `protected`, `content_position`, `content_type`,`contents_meta_title`,`contents_meta_description`,`contents_meta_keywords`) VALUES ";
    $languages     = $this->getLanguages();

    foreach ($languages as $language) {
        if ($language['languages_id'] !== '2') {
            $createContent .= "(0, 0, '" . $contentEn['group_ids'] . "', " . $language['languages_id']
                              . ", 'Free Content 1 (Malibu)', 'Free Content 1 (Malibu)', '', '"
                              . $this->real_escape_string($contentEn['content_text']) . "' , 0, 4, '', '', '"
                              . $contentEn['content_status']
                              . "', $nextContentGroupId, 0, '', '', '0.5', 'weekly', 0, 0, '', '1', 'elements_start', 'content','','',''),";
        } else {
            $createContent .= "(0, 0, '" . $contentDe['group_ids'] . "', " . $language['languages_id']
                              . ", 'Freier Inhalt 1 (Malibu)', 'Freier Inhalt 1 (Malibu)', '', '"
                              . $this->real_escape_string($contentDe['content_text']) . "', 0, 4, '', '', '"
                              . $contentDe['content_status']
                              . "', $nextContentGroupId, 0, '', '', '0.5', 'weekly', 0, 0, '', '1', 'elements_start', 'content','','',''),";
        }
    }

    // remove last ,
    $createContent = substr($createContent, 0, -1);

    $t_success &= $this->query($createContent, true);
    
    $createAliasQuery = 'REPLACE INTO `content_manager_aliases` (`content_group`, `content_alias`) VALUES (' . $nextContentGroupId . ', "Malibu-below-slider");';
    $t_success &= $this->query($createAliasQuery, true);
}

$t_success &= $this->set_index('orders', 'UNIQUE', array('date_purchased', 'orders_id'), 'date_purchased_orders_id');

$t_success &= $this->drop_index('gm_gprint_cart_elements', 'customers_id_products_id');
$t_success &= $this->query('ALTER TABLE `gm_gprint_cart_elements` ADD INDEX `customers_id_products_id`(`customers_id`, `products_id`(10))');

$t_success &= $this->drop_index('properties_description', 'properties_id');
$t_success &= $this->set_index('properties_description', 'UNIQUE', array('properties_id', 'language_id'), 'unique_description');


$checkIfHeaderOptionExists = $this->query('SELECT * FROM `gx_configurations` WHERE `key` = "gm_configuration/SEND_X_FRAME_OPTIONS_SAMEORIGIN_HEADER"', true);
if ($checkIfHeaderOptionExists->num_rows < 1) {
    $query = 'INSERT INTO `gx_configurations` SET `key` = "gm_configuration/SEND_X_FRAME_OPTIONS_SAMEORIGIN_HEADER", `value` = "true", `default` = "true", `type` = "switcher"';
    $t_success &= $this->query($query, true);
}


$index_array = [
    'container_type',
    'container_id',
    'addon_key'
];

if (!$this->indexExists('addon_values_storage', 'UNIQUE', $index_array)) {
    $this->set_index('addon_values_storage', 'UNIQUE', $index_array, 'container_type_container_id_addon_key');
}


/**
 * Changes the current value for pre existent variants
 * Note: Just in case of changing (updating) the layout of existing variants
 *
 * 1. We're checking if the shop, when updates, has an active HEADER 2 variant
 * 2. If the shop is using the variant we must change its values from theme and variants (settings.json)
 *
 */

/**
 * Helper function
 * Changes the values of the settings.json by the values passed by parameter $settingsArray
 *
 * @param $path
 * @param $settingsArray
 *
 * @return bool|false|int
 */
function changeSettingsValue($path, $settingsArray)
{
    $changed = false;

    if(file_exists($path))
    {
        $settingsJson = json_decode(file_get_contents($path), true);

        foreach($settingsJson as $index => &$setting)
        {
            if(array_key_exists($setting['name'], $settingsArray))
            {
                $setting = array_merge($settingsJson[$index], $settingsArray[$setting['name']]);
            }
        }

        unset($setting);

        $updatedSettingsJson = json_encode($settingsJson, JSON_PRETTY_PRINT);
        $changed             = file_put_contents($path, $updatedSettingsJson);
    }

    return $changed;
}

/**
 * Update Malibu only once, if shop is updated from version lower than 4.0
 */
$check = $this->query('SELECT * FROM `version_history` WHERE `version` > "4.0" AND `installation_date` < "'
                      . date('Y-m-d H:i:s', strtotime('-3 HOURS')) . '"', true);
if($check->num_rows < 1)
{
    $malibuSettingsJsonPath = dirname(__DIR__, 3) . '/themes/Malibu/settings.json';
    if(file_exists($malibuSettingsJsonPath))
    {
        $settingsJson = json_decode(file_get_contents($malibuSettingsJsonPath), true);
        $updateNeeded = false;

        // Checks if the header2 variant is active
        $filtered = array_filter($settingsJson, function ($element) {
            return ($element['name'] === 'header-variant' && $element['value']['id'] === 'header2');
        });

        if(count($filtered))
        {
            // Set the changes that we want to do
            $settingsToBeChanged = [
                // option ID            => values we want to change: type, group, value, etc
                'gx-hide-cart-col'      => ['value' => true],
                'gx-hide-secondary-col' => ['value' => false]
            ];

            // If the file exists and the change was made we change the variant's settings.json
            if(changeSettingsValue($malibuSettingsJsonPath, $settingsToBeChanged))
            {
                $malibuHeader2SettingsJsonPath = dirname(__DIR__, 3)
                                                 . '/themes/Malibu/variants/header-variant/header2/settings.json';
                changeSettingsValue($malibuHeader2SettingsJsonPath, $settingsToBeChanged);
            }
        }
    }
}

if ($this->table_exists('configuration')) {
    $query = "UPDATE `gx_configurations` a 
	            JOIN `configuration` b 
	            ON 
	                a.`key` = CONCAT('configuration/', b.`configuration_key`) AND
                    b.`set_function` = 'gm_cfg_select_option(array(''true'', ''false''),' 
                SET a.`type` = 'switcher'";
    $t_success &= is_numeric($this->query($query));

    $query = "UPDATE `gx_configurations`
                SET `default` = `value` 
                WHERE 
                    `type` = 'switcher' AND 
                     `default` IS NULL";
    $t_success &= is_numeric($this->query($query));

    $query = "UPDATE `gx_configurations` a 
	            JOIN `configuration` b 
	            ON 
	                a.`key` = CONCAT('configuration/', b.`configuration_key`) AND
                    b.`set_function` = 'xtc_cfg_select_option(array(''None'', ''Standard'', ''Credit Note''),' 
                SET 
                    a.`type` = 'tax-calculation-mode',
	                a.`default` = 'Standard'";
    $t_success &= is_numeric($this->query($query));

    $query = "UPDATE `gx_configurations` a 
	            JOIN `configuration` b 
	            ON 
	                a.`key` = CONCAT('configuration/', b.`configuration_key`) AND
                    b.`set_function` = 'xtc_cfg_select_option(array(''national'', ''international'', ''both''),' 
                SET 
                    a.`type` = 'shipping-destination', 
	                a.`default` = 'national'";
    $t_success &= is_numeric($this->query($query));

    $query = "UPDATE `gx_configurations` a 
	            JOIN `configuration` b 
	            ON 
	                a.`key` = CONCAT('configuration/', b.`configuration_key`) AND
                    b.`set_function` = 'xtc_cfg_pull_down_tax_classes(' 
                SET 
                    a.`type` = 'tax-class', 
                    a.`default` = '0'";
    $t_success &= is_numeric($this->query($query));

    $query = "UPDATE `gx_configurations` a 
	            JOIN `configuration` b 
	            ON 
	                a.`key` = CONCAT('configuration/', b.`configuration_key`) AND
                    b.`set_function` = 'xtc_cfg_pull_down_zone_classes(' 
                SET 
                    a.`type` = 'geo-zone'";
    $t_success &= is_numeric($this->query($query));

    $query = "UPDATE `gx_configurations` a 
	            JOIN `configuration` b 
	            ON 
	                a.`key` = CONCAT('configuration/', b.`configuration_key`) AND
                    b.`set_function` = 'xtc_cfg_pull_down_order_statuses(' 
                SET 
                    a.`type` = 'order-status'";
    $t_success &= is_numeric($this->query($query));
}

$query = 'ALTER TABLE `emails` ROW_FORMAT=DYNAMIC;';
$t_success &= $this->query($query, true);

$this->set_index('emails', 'INDEX', ['creation_date'], 'emails_creation_date_index');
$this->set_index('emails', 'INDEX', ['sent_date'], 'emails_sent_date_index');
$this->set_index('emails', 'INDEX', ['subject'], 'emails_subject_index');
$this->set_index('email_contacts', 'INDEX', ['email_address'], 'email_contacts_email_address_index');
